import tkinter as tk
from tkinter import filedialog
import random
import os

def main():
    while True:
        # asks the user how they want to make a graph
        print("Choose how you want to create the graph:")
        print("1. Load from file")
        print("2. Enter coordinates manually")
        print("3. Generate random coords")
        choice = input("Enter 1, 2, or 3: ")

        data_points = []

        if choice == "1":
            # Opens File Explorer and let the user pick the file
            root = tk.Tk()
            root.withdraw()  
            file_path = filedialog.askopenfilename(
                title="Select a file with x and y coordinates",
                filetypes=[("Text files", "*.txt"), ("All files", "*.*")]
            )

            # Check if a file was selected
            if not file_path:
                print("No file selected.")
                continue

            # Read data points from file
            try:
                with open(file_path, 'r') as file:
                    lines = file.readlines()
                    for line_number, line in enumerate(lines, start=1):
                        parts = line.strip().split()
                        # Ensure each line has exactly two parts
                        if len(parts) != 2:
                            print(f"Warning: Skipping line {line_number} due to incorrect format: '{line.strip()}'")
                            continue
                        try:
                            # Convert parts to integers
                            x = int(parts[0])
                            y = int(parts[1])
                            data_points.append((x, y))
                        except ValueError:
                            # error handling
                            print(f"Warning: Skipping line {line_number} due to non-integer values: '{line.strip()}'")
                            continue
            except FileNotFoundError:
                print("Error: The specified file was not found.")
                continue
            except Exception as e:
                print(f"Error reading file: {e}")
                continue

        elif choice == "2":
            # Allows the user to manually enter coordinates
            print("Enter coordinates (x y). Type 'done' to finish:")
            while True:
                user_input = input("Enter coordinate: ")
                if user_input.lower() == 'done':
                    break
                parts = user_input.strip().split()
                if len(parts) != 2: # error handling
                    print("Invalid input. Please enter two integers separated by a space.")
                    continue
                try:
                    x = int(parts[0])
                    y = int(parts[1])
                    data_points.append((x, y))
                except ValueError: # error handling
                    print("Invalid input. Please enter two integers.")
                    continue

            # code to maintain a continuos line between data points if there are some missing when the user inputs their own.
            # Finds a midpoint between values to find missing data points
            # This solution is probably not verry accurute but it is a simple and fast way to add in missing points
            data_points.sort()  # Sort data points by x value
            new_data_points = []
            for i in range(len(data_points) - 1):
                new_data_points.append(data_points[i])
                x1, y1 = data_points[i]
                x2, y2 = data_points[i + 1]
                # If there's a gap of more than 1 in the x values, add midpoint
                if x2 - x1 > 1:
                    mid_x = (x1 + x2) // 2
                    mid_y = (y1 + y2) / 2
                    new_data_points.append((mid_x, mid_y))
            new_data_points.append(data_points[-1])
            data_points = new_data_points

        elif choice == "3":
            # Generate random coordinates
            try:
                num_points = int(input("Enter the number of random points to generate: "))
                for _ in range(num_points):
                    x = random.randint(1, 20)
                    y = random.randint(-10, 10)
                    data_points.append((x, y))
            except ValueError:
                print("Invalid input. Please enter an integer value for the number of points.")
                continue

        else:
            print("Invalid choice. Please enter 1, 2, or 3.")
            continue

        # To check if there are valid data points
        if not data_points:
            print("No valid data points available.")
            continue

        # Makes the size of the graph dynamic based on the given data points
        try:
            max_y = max(y for _, y in data_points)
            min_y = min(y for _, y in data_points)
            max_x = max(x for x, _ in data_points)
        except ValueError:
            print("Error: Unable to determine graph dimensions due to lack of valid data points.")
            continue

        # Calculate the height and width of the graph
        height = max_y - min_y + 1
        width = max_x  # Adjust width to match maximum x value
        j = max_y

        # Draws the graph in the console with y axis labels
        for y in range(height):
            line = ""

            #label y axis
            if j > 0:
                line += f" {j}|"
            elif j == 0:
                line += " 0+"
            else:
                line += f"{j}|"

            #loop through each x position
            for x in range(1, width + 1):
                if (x, j) in data_points:
                    line += "x" #mark data points with x
                elif j == 0:
                    line += "-" # for the x axis line
                else:
                    line += " " #Empty space for other area

            print(line)
            j -= 1 #deincrement for next row

        print()

        # Ask user if they want to save the graph
        save_choice = input("Do you want to save the graph to a file? (yes / any key for no): ").lower()
        if save_choice != 'yes':
            print("Graph not saved.")
            break

        # Open File Explorer to save the graph to a file
        save_path = filedialog.asksaveasfilename(
            title="Save graph to file",
            defaultextension=".txt",
            filetypes=[("Text files", "*.txt"), ("All files", "*.*")]
        )

        # Check if a save location was selected
        if not save_path:
            print("No file selected for saving.")
            continue

        # Write the graph to the specified file
        try:
            with open(save_path, 'w') as output_file:
                j = max_y
                for y in range(height):
                    line = ""
                    if j > 0: #lebel x axis
                        line += f" {j}|"
                    elif j == 0:
                        line += " 0+"
                    else:
                        line += f"{j}|" # for negative y values

                    for x in range(1, width + 1):
                        if (x, j) in data_points:
                            line += "x" #makrs data points
                        elif j == 0:
                            line += "-" # x axis line
                        else:
                            line += " " # empty space

                    output_file.write(line + "\n")
                    j -= 1

                output_file.write("\n")
            print(f"Graph saved to {save_path}")
        except Exception as e:
            print(f"Error saving file: {e}")
        break

# Calls the main function
if __name__ == "__main__":
    main()
